@@ -20,24 +20,24 @@ class ProfileEditorUI : public QFrame, public Ui::ProfileEditorUI
 {
     Q_OBJECT
 
-    public:
+  public:
     /** @short Constructor
-     */
-        explicit ProfileEditorUI( QWidget *parent );
+         */
+    explicit ProfileEditorUI(QWidget *parent);
 };
 
 class ProfileEditor : public QDialog
 {
     Q_OBJECT
-public:
+  public:
     /**
-     *@short Constructor.
-     */
-    explicit ProfileEditor(QWidget *ks );
+         *@short Constructor.
+         */
+    explicit ProfileEditor(QWidget *ks);
 
     /**
-     *@short Destructor.
-     */
+         *@short Destructor.
+         */
     ~ProfileEditor();
 
     ProfileInfo *getPi() const;
@@ -45,15 +45,23 @@ public:
 
     void loadDrivers();
 
-public slots:
+    void setProfileName(const QString &name);
+    void setAuxDrivers(const QStringList &aux);
+    void setHostPort(const QString &host, const QString &port);
+    void setWebManager(bool enabled, const QString &port = "8624");
+    void setGuiderType(int type);
+    void setConnectionOptionsEnabled(bool enable);
+
+  public slots:
     void saveProfile();
     void setRemoteMode(bool enable);
 
-private:
+  private slots:
+    void updateGuiderSelection(int id);
 
+  private:
     ProfileEditorUI *ui;
     ProfileInfo *pi;
-
 };
 
 #endif // PROFILEEDITOR_H
