@@ -26,7 +26,6 @@
 #include <QStatusBar>
 #include <QStandardPaths>
 
-
 #include "servermanager.h"
 #include "drivermanager.h"
 #include "driverinfo.h"
@@ -34,17 +33,14 @@
 #include "Options.h"
 #include "kstars.h"
 #include "kstarsdatetime.h"
-
-
-const int INDI_MAX_TRIES=3;
-const int MAX_FILENAME_LEN = 1024;
+#include "kspaths.h"
 
 ServerManager::ServerManager(QString inHost, uint inPort)
 {
-    serverProcess	= NULL;
-    XMLParser		= NULL;
-    host		= inHost;
-    port		= QString::number(inPort);
+    serverProcess = nullptr;
+    XMLParser     = nullptr;
+    host          = inHost;
+    port          = QString::number(inPort);
 
     //qDebug() << "We got port unit with value of " << inPort << " and now as tring it is equal to #" << port << "#" << endl;
 }
@@ -68,21 +64,50 @@ bool ServerManager::start()
     qWarning() << "INDI server is currently not supported on Windows.";
     return false;
 #else
-    bool connected=false;
-    int fd=0;
+    bool connected = false;
+    int fd         = 0;
 
-    if (serverProcess == NULL)
-        serverProcess = new QProcess(this);
+    if (serverProcess == nullptr)
+    {
+        serverProcess      = new QProcess(this);
+#ifdef Q_OS_OSX
+        QString driversDir = Options::indiDriversDir();
+        if (Options::indiDriversAreInternal())
+            driversDir = QCoreApplication::applicationDirPath() + "/indi";
+        QString indiServerDir = Options::indiServer();
+        if (Options::indiServerIsInternal())
+            indiServerDir = QCoreApplication::applicationDirPath() + "/indi";
+        else
+            indiServerDir = QFileInfo(Options::indiServer()).dir().path();
+        QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+        env.insert("PATH", driversDir + ":" + indiServerDir + ":/usr/local/bin:/usr/bin:/bin");
+        QString gscDirPath = KSPaths::writableLocation(QStandardPaths::GenericDataLocation) + "gsc";
+        env.insert("GSCDAT", gscDirPath);
+        QString gphoto_iolibs = QCoreApplication::applicationDirPath() + "/../PlugIns/libgphoto2_port";
+        if (QFileInfo(gphoto_iolibs).exists() && Options::indiDriversAreInternal())
+            env.insert("IOLIBS", QDir(gphoto_iolibs).absolutePath());
+        QString gphoto_camlibs = QCoreApplication::applicationDirPath() + "/../PlugIns/libgphoto2";
+        if (QFileInfo(gphoto_camlibs).exists() && Options::indiDriversAreInternal())
+            env.insert("CAMLIBS", QDir(gphoto_camlibs).absolutePath());
+        QString qhyFirmwarePath = QCoreApplication::applicationDirPath() + "/../PlugIns/qhy";
+        if (QFileInfo(qhyFirmwarePath).exists() && Options::indiDriversAreInternal())
+            env.insert("QHY_FIRMWARE_DIR", QDir(qhyFirmwarePath).absolutePath());
+        env.insert("INDIPREFIX", indiServerDir);
+        serverProcess->setProcessEnvironment(env);
+#endif
+    }
 
     QStringList args;
 
-    args << "-v" << "-p" << port << "-m" << "100";
+    args << "-v"
+         << "-p" << port << "-m"
+         << "100";
 
     QString fifoFile = QString("/tmp/indififo%1").arg(QUuid::createUuid().toString().mid(1, 8));
 
-    if ( (fd = mkfifo (fifoFile.toLatin1(), S_IRUSR| S_IWUSR) < 0))
+    if ((fd = mkfifo(fifoFile.toLatin1(), S_IRUSR | S_IWUSR) < 0))
     {
-        KMessageBox::error(NULL, i18n("Error making FIFO file %1: %2.", fifoFile, strerror(errno)));
+        KMessageBox::error(nullptr, i18n("Error making FIFO file %1: %2.", fifoFile, strerror(errno)));
         return false;
     }
 
@@ -96,19 +121,25 @@ bool ServerManager::start()
         return false;
     }
 
-    args << "-f" << fifoFile;    
+    args << "-f" << fifoFile;
 
     serverProcess->setProcessChannelMode(QProcess::SeparateChannels);
     serverProcess->setReadChannel(QProcess::StandardError);
 
-    serverProcess->start(Options::indiServer(), args);
+#ifdef Q_OS_OSX
+    if (Options::indiServerIsInternal())
+        serverProcess->start(QCoreApplication::applicationDirPath() + "/indi/indiserver", args);
+    else
+#endif
+        serverProcess->start(Options::indiServer(), args);
 
     connected = serverProcess->waitForStarted();
 
     if (connected)
     {
-        connect(serverProcess, SIGNAL(error(QProcess::ProcessError)), this, SLOT(processServerError(QProcess::ProcessError)));
-        connect(serverProcess, SIGNAL(readyReadStandardError()),  this, SLOT(processStandardError()));
+        connect(serverProcess, SIGNAL(error(QProcess::ProcessError)), this,
+                SLOT(processServerError(QProcess::ProcessError)));
+        connect(serverProcess, SIGNAL(readyReadStandardError()), this, SLOT(processStandardError()));
 
         emit started();
     }
@@ -130,17 +161,17 @@ bool ServerManager::startDriver(DriverInfo *dv)
     // Check for duplicates within managed drivers
     if (dv->getUniqueLabel().isEmpty() == false)
     {
-        int nset=0;
+        int nset = 0;
         QString uniqueLabel;
         QString label = dv->getUniqueLabel();
-        foreach(DriverInfo *drv, managedDrivers)
+        foreach (DriverInfo *drv, managedDrivers)
         {
             if (label == drv->getUniqueLabel())
                 nset++;
         }
         if (nset > 0)
         {
-            uniqueLabel = QString("%1 %2").arg(label).arg(nset+1);
+            uniqueLabel = QString("%1 %2").arg(label).arg(nset + 1);
             dv->setUniqueLabel(uniqueLabel);
         }
     }
@@ -148,26 +179,47 @@ bool ServerManager::startDriver(DriverInfo *dv)
     managedDrivers.append(dv);
     dv->setServerManager(this);
 
+    QString driversDir    = Options::indiDriversDir();
+    QString indiServerDir = Options::indiServer();
+
+#ifdef Q_OS_OSX
+    if (Options::indiServerIsInternal())
+        driversDir = QCoreApplication::applicationDirPath() + "/indi";
+    if (Options::indiDriversAreInternal())
+        indiServerDir = QCoreApplication::applicationDirPath() + "/indi";
+    else
+        indiServerDir = QFileInfo(Options::indiServer()).dir().path();
+#endif
+
+    QStringList paths;
+    paths << "/usr/bin"
+          << "/usr/local/bin" << driversDir << indiServerDir;
+
     if (QStandardPaths::findExecutable(dv->getDriver()).isEmpty())
     {
-         KMessageBox::error(NULL, i18n("Driver %1 was not found on the system. Please make sure the package that provides the '%1' binary is installed.", dv->getDriver()));
-         return false;
+        if (QStandardPaths::findExecutable(dv->getDriver(), paths).isEmpty())
+        {
+            KMessageBox::error(nullptr, i18n("Driver %1 was not found on the system. Please make sure the package that "
+                                             "provides the '%1' binary is installed.",
+                                             dv->getDriver()));
+            return false;
+        }
     }
 
-        out << "start " << dv->getDriver();
-        if (dv->getUniqueLabel().isEmpty() == false)
-            out << " -n \"" << dv->getUniqueLabel() << "\"";
-        if (dv->getSkeletonFile().isEmpty() == false)
-            out << " -s \"" << Options::indiDriversDir() << QDir::separator() << dv->getSkeletonFile() << "\"";
-        out << endl;
+    out << "start " << dv->getDriver();
+    if (dv->getUniqueLabel().isEmpty() == false)
+        out << " -n \"" << dv->getUniqueLabel() << "\"";
+    if (dv->getSkeletonFile().isEmpty() == false)
+        out << " -s \"" << driversDir << QDir::separator() << dv->getSkeletonFile() << "\"";
+    out << endl;
 
-        out.flush();
+    out.flush();
 
-        dv->setServerState(true);
+    dv->setServerState(true);
 
-        dv->setPort(port);
+    dv->setPort(port);
 
-        return true;
+    return true;
 }
 
 void ServerManager::stopDriver(DriverInfo *dv)
@@ -177,7 +229,7 @@ void ServerManager::stopDriver(DriverInfo *dv)
     managedDrivers.removeOne(dv);
 
     if (dv->getUniqueLabel().isEmpty() == false)
-       out << "stop " << dv->getDriver() << " -n \"" << dv->getUniqueLabel() << "\"" << endl;
+        out << "stop " << dv->getDriver() << " -n \"" << dv->getUniqueLabel() << "\"" << endl;
     else
         out << "stop " << dv->getDriver() << endl;
 
@@ -188,7 +240,7 @@ void ServerManager::stopDriver(DriverInfo *dv)
 
 bool ServerManager::isDriverManaged(DriverInfo *di)
 {
-    foreach(DriverInfo *dv, managedDrivers)
+    foreach (DriverInfo *dv, managedDrivers)
     {
         if (dv == di)
             return true;
@@ -199,10 +251,10 @@ bool ServerManager::isDriverManaged(DriverInfo *di)
 
 void ServerManager::stop()
 {
-    if (serverProcess == NULL)
+    if (serverProcess == nullptr)
         return;
 
-    foreach(DriverInfo *device, managedDrivers)
+    foreach (DriverInfo *device, managedDrivers)
     {
         device->setServerState(false);
         device->clear();
@@ -216,13 +268,12 @@ void ServerManager::stop()
 
     delete serverProcess;
 
-    serverProcess = NULL;
-
+    serverProcess = nullptr;
 }
 
 void ServerManager::terminate()
 {
-    if (serverProcess == NULL)
+    if (serverProcess == nullptr)
         return;
 
     serverProcess->terminate();
@@ -232,54 +283,56 @@ void ServerManager::terminate()
 
 void ServerManager::connectionSuccess()
 {
+    foreach (DriverInfo *device, managedDrivers)
+        device->setServerState(true);
 
-    foreach(DriverInfo *device, managedDrivers)
-         device->setServerState(true);
-
-    connect(serverProcess, SIGNAL(readyReadStandardError()),  this, SLOT(processStandardError()));
+    connect(serverProcess, SIGNAL(readyReadStandardError()), this, SLOT(processStandardError()));
 
     emit started();
 }
 
 void ServerManager::processServerError(QProcess::ProcessError err)
 {
-  INDI_UNUSED(err);
-  emit serverFailure(this);
+    INDI_UNUSED(err);
+    emit serverFailure(this);
 }
 
 void ServerManager::processStandardError()
 {
-    #ifdef Q_OS_WIN
+#ifdef Q_OS_WIN
     qWarning() << "INDI server is currently not supported on Windows.";
     return;
-    #else
+#else
     QString stderr = serverProcess->readAllStandardError();
 
     serverBuffer.append(stderr);
 
     if (Options::iNDILogging())
     {
-        foreach(QString msg, stderr.split("\n"))
+        foreach (QString msg, stderr.split("\n"))
             qDebug() << "INDI Server: " << msg;
     }
 
     if (driverCrashed == false && (serverBuffer.contains("stdin EOF") || serverBuffer.contains("stderr EOF")))
     {
         QStringList parts = serverBuffer.split("Driver");
-        foreach(QString driver, parts)
+        foreach (QString driver, parts)
         {
             if (driver.contains("stdin EOF") || driver.contains("stderr EOF"))
             {
-                driverCrashed=true;
+                driverCrashed      = true;
                 QString driverName = driver.left(driver.indexOf(':')).trimmed();
-                KMessageBox::information(0, i18n("KStars detected INDI driver %1 crashed. Please check INDI server log in the Device Manager.", driverName));
+                KMessageBox::information(
+                    0,
+                    i18n("KStars detected INDI driver %1 crashed. Please check INDI server log in the Device Manager.",
+                         driverName));
                 break;
             }
         }
-   }
+    }
 
     emit newServerLog();
-    #endif
+#endif
 }
 
 QString ServerManager::errorString()
@@ -287,7 +340,5 @@ QString ServerManager::errorString()
     if (serverProcess)
         return serverProcess->errorString();
 
-    return NULL;
+    return nullptr;
 }
-
-
